unit Schemes;

interface

uses Basis,Differences;

function Walkthrow(a,b,c,f :TVector; k1,k2,m1,m2 :Float) :TVector;
function UnrealScheme :TMatrix;
function RealScheme :TMatrix;
function Delta(const A :TMatrix) :Float;

implementation

uses SysUtils,Dialogs;

function Walkthrow(a,b,c,f :TVector; k1,k2,m1,m2 :Float) :TVector;
var
  alfa,beta :TVector;
  z :Float;
  k :Integer;
begin
  alfa[0] := k1;
  beta[0] := m1;
  for k := 0 to n-1 do begin
    z := b[k] + c[k]*alfa[k];
    alfa[k+1] := -a[k]/z;
    beta[k+1] := (f[k]-c[k]*beta[k])/z;
  end;
  Result[n] := (k2*beta[n]+m2)/(1-k2*alfa[n]);
  for k := n downto 1 do begin
    Result[k-1] := alfa[k]*Result[k] + beta[k];
  end;
end;

function RealScheme :TMatrix;
var
  a,a1 :Float;
  i,j :Integer;
begin
  for i := 0 to n do Result[0,i] := u0(i);
  for j := 0 to m do begin
    Result[j,0] := mu1(j);
    Result[j,n] := mu2(j);
  end;
  for j := 0 to m-1 do begin
    for i := 1 to n-1 do begin
      a := afunc(i,j);
      a1 := afunc(i+1,j);
      Result[j+1,i] := hh*(a1*(Result[j,i+1]-Result[j,i]) - a*(Result[j,i]-Result[j,i-1])) + Result[j,i] + h2*phifunc(i,j);
    end;
  end;
end;

function UnrealScheme :TMatrix;
var
  a,b,c,f :TVector;
  i,j :Integer;
begin
  for i := 0 to n do Result[0,i] := u0(i);
  for j := 0 to m-1 do begin
    for i := 0 to n do begin
      a[i] := afunc(i,j);
      c[i] := afunc(i+1,j);
      f[i] := -(Result[j,i] + h2*phifunc(i,j));
      a[i] := hh*a[i];
      c[i] := hh*c[i];
      b[i] := -(a[i]+c[i]+1);
    end;
    Result[j+1] := Walkthrow(a,b,c,f,0,0,mu1(j),mu2(j));
  end;
end;
  {
  a := (k(x(i-1),y(j)) + k(x(i),y(j)))/2;
  a1 := (k(x(i),y(j)) + k(x(i+1),y(j)))/2;
  p := (f(x(i),y(j)) + f(x(i),y(j+1)))/2;
  }

function Delta(const A :TMatrix) :Float;
var
  d :Float;
  i,j :Integer;
begin
  Result := 0;
  for j := 0 to m do begin
    for i := 0 to n do begin
      d := abs(A[j,i] - Exact(i,j));
      if Result<d then Result := d;
    end;
  end;
end;

end.
